/*
								+---------------------------------+
								|                                 |
								| ***  Multi-mon win support  *** |
								|                                 |
								|  Copyright   -tHE SWINe- 2006  |
								|                                 |
								|           Monitors.h            |
								|                                 |
								+---------------------------------+
*/

#ifndef __MULTI_MON_INCLUDED
#define __MULTI_MON_INCLUDED

#include <multimon.h>

/**
 *	@brief information about monitor devices
 *
 *	Provides information about resolution, position, refresh rate,
 *		etc. of all monitors, attached to the system.
 *
 *	@note This can only be compiled on windows.
 */
class CMultimonInfo {
public:
	/**
	 *	@brief monitor info
	 */
	struct TMonitor {
		bool b_primary; /**< primary monitor flag */
		int n_x, /**< monitor upper-left corner x (in virtual screen coordinates) */
			n_y, /**< monitor upper-left corner y (in virtual screen coordinates) */
			n_width, /**< monitor width in pixels */
			n_height; /**< monitor height in pixels */
		int n_bit_depth; /**< number of bits per pixel */
		int n_refresh_rate; /**< refresh rate, in Hz */
		char p_s_name[128]; /**< device name */
	};

private:
	std::vector<TMonitor> m_monitor_list;
	TMonitor m_t_full_window;

public:
	/**
	 *	@brief default constructor
	 *
	 *	Updates information about monitors. In case error occured, number
	 *		of monitors is set to zero (n_Monitor_Num() returns 0).
	 */
	CMultimonInfo();

	/**
	 *	@brief updates information about monitors
	 *
	 *	Updates information about monitors, use in case number of monitors
	 *		connected to the system / their configuration may have changed.
	 *		Otherwise, information about monitors is retrieved in constructor
	 *		already.
	 *
	 *	@return Returns true on success, false on failure.
	 */
	bool Update_Monitors();

	/**
	 *	@brief gets index of primary monitor
	 *
	 *	@return Returns zero-based index of primary monitor, or -1 in case
	 *		primary monitor was not found.
	 */
	int n_PrimaryMonitor() const;

	/**
	 *	@brief gets monitor count
	 *
	 *	@return Returns number of monitors, connected to the system.
	 */
	inline int n_Monitor_Num() const
	{
		return m_monitor_list.size();
	}

	/**
	 *	@brief gets info about a monitor
	 *
	 *	Gets information about monitor with zero-based index n_index.
	 *
	 *	@param[in] n_index is zero-based index of monitor
	 *
	 *	@return Returns information about the selected monitor.
	 */
	inline const TMonitor &t_Monitor(int n_index) const
	{
		return m_monitor_list[n_index];
	}

	/**
	 *	@brief gets info about virtual screen
	 *
	 *	Virtual screen is bounding box of all monitors, connected to the system.
	 *		Use this to create windows, spanning all the monitors.
	 *
	 *	@note Bit depth and refresh rate of virtual screen is not available and
	 *		is set to zero (it is possible to query bit depths / refresh rates
	 *		of all the monitors and choose either minimum, maximum or the first one).
	 */
	inline const TMonitor &t_Virtual_Screen() const
	{
		return m_t_full_window;
	}

private:
	static inline bool b_IsPrimary(const TMonitor &r_t_mon) { return r_t_mon.b_primary; }
	static BOOL CALLBACK _MonCallback(HMONITOR h_monitor, HDC h_dc, LPRECT p_rect, LPARAM n_l_param);
	bool MonCallback(HMONITOR h_monitor, LPRECT p_rect);
};

#endif //__MULTI_MON_INCLUDED
